@@ -7,11 +7,13 @@ import os
 import errno
 import unicodedata
 import getpass
+import netrc
 import mechanize
 import cookielib
 from bs4 import BeautifulSoup
 import tempfile
 from os import path
+import platform
 import _version
 
 class CourseraDownloader(object):
@@ -37,6 +39,9 @@ class CourseraDownloader(object):
     #see http://www.crummy.com/software/BeautifulSoup/bs4/doc/#installing-a-parser
     DEFAULT_PARSER = "lxml"
 
+    # how long to try to open a URL before timing out
+    TIMEOUT=60.0
+
     def __init__(self,username,password,proxy=None,parser=DEFAULT_PARSER,ignorefiles=None):
         self.username = username
         self.password = password
@@ -140,7 +145,7 @@ class CourseraDownloader(object):
         print "* Collecting downloadable content from " + course_url
 
         # get the course name, and redirect to the course lecture page
-        vidpage = self.browser.open(course_url)
+        vidpage = self.browser.open(course_url,timeout=self.TIMEOUT)
 
         # extract the weekly classes
         soup = BeautifulSoup(vidpage,self.parser)
@@ -204,8 +209,8 @@ class CourseraDownloader(object):
                 if not hasvid:
                     ll = li.find('a',{'class':'lecture-link'})
                     lurl = ll['data-modal-iframe']
-                    bb = self.browser.open(lurl)
-                    bb = BeautifulSoup(p,self.parser)
+                    pg = self.browser.open(lurl,timeout=self.TIMEOUT)
+                    bb = BeautifulSoup(pg,self.parser)
                     vobj = bb.find('source',type="video/mp4")
 
                     if not vobj:
@@ -229,7 +234,7 @@ class CourseraDownloader(object):
         """
         Get the headers
         """
-        r = self.browser.open(url)
+        r = self.browser.open(url,timeout=self.TIMEOUT)
         return r.info()
 
     def download(self, url, target_dir=".", target_fname=None):
@@ -279,7 +284,7 @@ class CourseraDownloader(object):
 
         try:
            if dl:
-                self.browser.retrieve(url,filepath)
+                self.browser.retrieve(url,filepath,timeout=self.TIMEOUT)
         except Exception as e:
             print "Failed to download url %s to %s: %s" % (url,filepath,e)
 
@@ -288,7 +293,7 @@ class CourseraDownloader(object):
         Download all the contents (quizzes, videos, lecture notes, ...) of the course to the given destination directory (defaults to .)
         """
         # open the main class page
-        self.browser.open(self.AUTH_URL % cname)
+        self.browser.open(self.AUTH_URL % cname,timeout=self.TIMEOUT)
 
         # get the lecture url
         course_url = self.lecture_url_from_name(cname)
@@ -494,6 +499,43 @@ class AbsoluteURLGen(object):
                     url = url[2:]
                 return self.base + url
 
+def get_netrc_creds():
+    """
+    Read username/password from the users' netrc file. Returns None if no
+    coursera credentials can be found.
+    """
+    # inspired by https://github.com/jplehmann/coursera
+
+    if platform.system() == 'Windows':
+        # where could the netrc file be hiding, try a number of places 
+        env_vars = ["HOME","HOMEDRIVE", "HOMEPATH","USERPROFILE","SYSTEMDRIVE"]
+        env_dirs = [os.environ[e] for e in env_vars if os.environ.get(e,None)]
+
+        # also try the root/cur dirs
+        env_dirs += ["C:", ""]
+
+        # possible filenames
+        file_names = [".netrc", "_netrc"]
+
+        # all possible paths
+        paths = [path.join(dir,fn) for dir in env_dirs for fn in file_names]
+    else:
+        # on *nix just put None, and the correct default will be used
+        paths = [None]
+
+    # try the paths one by one and return the first one that works
+    creds = None
+    for p in paths:
+        try:
+            auths = netrc.netrc(p).authenticators('coursera-dl')
+            creds = (auths[0], auths[2])
+            print "Credentials found in .netrc file"
+            break
+        except (IOError, TypeError, netrc.NetrcParseError) as e:
+            pass
+
+    return creds
+
 # is lxml available?
 def haslxml():
     try:
@@ -505,8 +547,8 @@ def haslxml():
 def main():
     # parse the commandline arguments
     parser = argparse.ArgumentParser(description='Download Coursera.org course videos/docs for offline use.')
-    parser.add_argument("-u", dest='username', type=str, required=True, help='coursera.org username')
-    parser.add_argument("-p", dest='password', type=str, help='coursera.org password')
+    parser.add_argument("-u", dest='username', type=str, help='coursera username (.netrc used if omitted)')
+    parser.add_argument("-p", dest='password', type=str, help='coursera password')
     parser.add_argument("-d", dest='dest_dir', type=str, default=".", help='destination directory where everything will be saved')
     parser.add_argument("-n", dest='ignorefiles', type=str, default="", help='comma-separated list of file extensions to skip, e.g., "ppt,srt,pdf"')
     parser.add_argument("-q", dest='parser', type=str, default=CourseraDownloader.DEFAULT_PARSER,
@@ -528,15 +570,25 @@ def main():
 
     print "Coursera-dl v%s (%s)" % (_version.__version__,parser)
 
-    # prompt the user for his password if not specified
-    if not args.password:
-        args.password = getpass.getpass()
+    # search for login credentials in .netrc file if username hasn't been provided in command-line args
+    username, password = args.username, args.password
+    if not username:
+        creds = get_netrc_creds()
+        if not creds:
+            raise Exception("No username passed and no .netrc credentials found, unable to login")
+        else:
+            username, password = creds
+    else:
+        # prompt the user for his password if not specified
+        if not password:
+            password = getpass.getpass()
 
     # instantiate the downloader class
-    d = CourseraDownloader(args.username,args.password,proxy=args.proxy,parser=parser,ignorefiles=args.ignorefiles)
+    d = CourseraDownloader(username,password,proxy=args.proxy,parser=parser,ignorefiles=args.ignorefiles)
     
     # authenticate, only need to do this once but need a classaname to get hold
     # of the csrf token, so simply pass the first one
+    print "Logging in as '%s'..." % username
     d.login(args.course_names[0])
 
     # download the content
